@@ -5,11 +5,13 @@ from rdflib.namespace import Namespace
 
 class ListRedirect(object):
     """
-    A utility class for lists of items joined by an operator.  ListRedirects with length 1
-    are a special case and are considered equivalent to the item instead of a list containing it.
-    The reduce function is used for normalizing ListRedirect to the single item (and calling reduce on it recursively)
+    A utility class for lists of items joined by an operator.  ListRedirects
+    with length 1 are a special case and are considered equivalent to the
+    item instead of a list containing it. The reduce function is used for
+    normalizing ListRedirect to the single item (and calling reduce on it
+    recursively)
     """
-    reducable = True
+    reducible = True
     def __getattr__(self, attr):
         if hasattr(self._list, attr):
             return getattr(self._list, attr)
@@ -23,7 +25,7 @@ class ListRedirect(object):
         return len(self._list)
 
     def reduce(self):
-        if self.reducable and len(self._list) == 1:
+        if self.reducible and len(self._list) == 1:
             singleItem = self._list[0]
             if isinstance(singleItem,ListRedirect):
                 return singleItem.reduce()
@@ -152,8 +154,9 @@ class ParsedString(unicode):
 class ParsedDatatypedLiteral(object):
     """
     Placeholder for Datatyped literals
-    This is neccessary (instead of instanciating Literals directly)
-    when datatypes IRIRefs are QNames (in which case the prefix needs to be resolved at some point)
+    This is necessary (instead of instantiating Literals directly)
+    when datatypes IRIRefs are QNames (in which case the prefix needs
+    to be resolved at some point)
     """
     def __init__(self,value,dType):
         self.value = value
@@ -313,15 +316,18 @@ class BlockOfTriples(object):
 
 class GraphPattern(object):
     """
-    Complex graph patterns can be made by combining simpler graph patterns. The ways of creating graph patterns are:
+    Complex graph patterns can be made by combining simpler graph patterns. 
+    The ways of creating graph patterns are:
     * Basic Graph Patterns, where a set of triple patterns must match
-    * Group Graph Pattern, where a set of graph patterns must all match using the same variable substitution
+    * Group Graph Pattern, where a set of graph patterns must all match using
+    the same variable substitution
     * Value constraints, which restrict RDF terms in a solution
     * Optional Graph patterns, where additional patterns may extend the solution
     * Alternative Graph Pattern, where two or more possible patterns are tried
     * Patterns on Named Graphs, where patterns are matched against named graphs
 
     ( GraphPatternNotTriples | Filter ) '.'? TriplesBlock?
+    
     """
     def __init__(self,nonTripleGraphPattern=None,filter=None,triples=None):
         #print "GraphPattern(..)",triples,filter,nonTripleGraphPattern
@@ -554,10 +560,15 @@ class DescribeQuery(object):
         self.solutionModifier = solutionModifier
 
     def __repr__(self):
+        if self.whereClause is not None:
+            parsedGP = self.whereClause.parsedGraphPattern
+        else:
+            parsedGP = None
         return "DESCRIBE %s %s %s %s"%(
                        self.describeVars,
                        self.dataSets,
-                       self.whereClause.parsedGraphPattern,
+                       parsedGP,
+                       # self.whereClause.parsedGraphPattern,
                        self.solutionModifier)
 
 
@@ -592,7 +603,8 @@ class RDFTerm(object):
 class Resource(RDFTerm):
     """
     Represents a sigle resource in a triple pattern.  It consists of an identifier
-    (URIRef or BNode) and a list of rdflib.sparql.bison.Triples.PropertyValue instances
+    (URIRef or BNode) and a list of :class:`~rdfextras.sparql.components.PropertyValue`
+    instances
     """
     def __init__(self,identifier=None,propertyValueList=None):
         self.identifier = identifier is not None and identifier or BNode()
@@ -625,7 +637,7 @@ class ParsedCollection(ListRedirect,RDFTerm):
     """
     An RDF Collection
     """
-    reducable = False
+    reducible = False
     def __init__(self,graphNodeList=None):
         self.propVals = []
         if graphNodeList:
@@ -694,6 +706,12 @@ class PropertyValue(object):
     def __eq__(self, other):
         return (self.property == other.property and
                 self.objects == other.objects)
+    
+    def __hash__(self):
+        h = hash(self.property)  
+        for o in self.objects:
+            h ^= hash(o)
+        return h
 
 class ParsedConstrainedTriples(object):
     """
